#pragma once
#include <glad/glad.h>
#include <JGL/types/Texture.h>
#include <Color4.hpp>
#include <Colors.hpp>

namespace JGL {
    class RenderTarget;
}

class JGL::RenderTarget {
private:
    Color4 clear_color{0,0,0,0};
    /// "Size" in this sense is the "Renderable Area" because OpenGL textures behave strangely if they're not square.
    Vector2 size{0, 0};
    bool using_depth = false;
    GLuint framebuffer_object = 0;
    GLuint depth_buffer = 0;
    Texture* texture = nullptr;
public:
    static GLuint GetActiveGLFramebufferHandle();
    static void SetActiveGLRenderTarget(const RenderTarget& render_target);
public:
    [[nodiscard]] Vector2 GetDimensions() const;
    [[nodiscard]] Texture* GetJGLTexture() const;
    [[nodiscard]] GLuint GetGLTextureHandle() const;
    [[nodiscard]] GLuint GetGLFramebufferObjectHandle() const;
    [[nodiscard]] GLuint GetGLDepthBufferHandle() const;
    [[nodiscard]] Color4 GetClearColor() const;
    /// Get the data back from the FBO. This is *not* async friendly.
    [[nodiscard]] std::vector<GLfloat> GetData() const;
public:
    /// Create a render target for a texture that already exists. For decals.
    explicit RenderTarget(const Texture& texture, const Color4& clear_color = Colors::Black, bool use_depth = false);
    /// Create a Render Target with a brand new texture. Want to render JGL elements onto a texture and display it as a sprite?
    explicit RenderTarget(const Vector2& size,  const Color4& clear_color = Colors::Black, bool use_depth = false);

    void Erase();
};